@@ -14,6 +14,7 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
+// Package config contains utilities for managing the kubeadm configuration API.
 package config
 
 import (
@@ -44,6 +45,14 @@ import (
 	kubeadmutil "k8s.io/kubernetes/cmd/kubeadm/app/util"
 )
 
+// LoadOrDefaultConfigurationOptions holds the common LoadOrDefaultConfiguration options.
+type LoadOrDefaultConfigurationOptions struct {
+	// AllowExperimental indicates whether the experimental / work in progress APIs can be used.
+	AllowExperimental bool
+	// SkipCRIDetect indicates whether to skip the CRI socket detection when no CRI socket is provided.
+	SkipCRIDetect bool
+}
+
 // MarshalKubeadmConfigObject marshals an Object registered in the kubeadm scheme. If the object is a InitConfiguration or ClusterConfiguration, some extra logic is run
 func MarshalKubeadmConfigObject(obj runtime.Object, gv schema.GroupVersion) ([]byte, error) {
 	switch internalcfg := obj.(type) {
@@ -242,9 +251,13 @@ func validateKnownGVKs(gvks []schema.GroupVersionKind) error {
 
 // MigrateOldConfig migrates an old configuration from a byte slice into a new one (returned again as a byte slice).
 // Only kubeadm kinds are migrated.
-func MigrateOldConfig(oldConfig []byte, allowExperimental bool) ([]byte, error) {
+func MigrateOldConfig(oldConfig []byte, allowExperimental bool, mutators migrateMutators) ([]byte, error) {
 	newConfig := [][]byte{}
 
+	if mutators == nil {
+		mutators = defaultMigrateMutators()
+	}
+
 	gvkmap, err := kubeadmutil.SplitYAMLDocuments(oldConfig)
 	if err != nil {
 		return []byte{}, err
@@ -265,10 +278,13 @@ func MigrateOldConfig(oldConfig []byte, allowExperimental bool) ([]byte, error)
 	}
 	// Migrate InitConfiguration and ClusterConfiguration if there are any in the config
 	if kubeadmutil.GroupVersionKindsHasInitConfiguration(gvks...) || kubeadmutil.GroupVersionKindsHasClusterConfiguration(gvks...) {
-		o, err := documentMapToInitConfiguration(gvkmap, true, allowExperimental, true)
+		o, err := documentMapToInitConfiguration(gvkmap, true, allowExperimental, true, false)
 		if err != nil {
 			return []byte{}, err
 		}
+		if err := mutators.mutate([]any{o}); err != nil {
+			return []byte{}, err
+		}
 		b, err := MarshalKubeadmConfigObject(o, gv)
 		if err != nil {
 			return []byte{}, err
@@ -278,10 +294,13 @@ func MigrateOldConfig(oldConfig []byte, allowExperimental bool) ([]byte, error)
 
 	// Migrate JoinConfiguration if there is any
 	if kubeadmutil.GroupVersionKindsHasJoinConfiguration(gvks...) {
-		o, err := documentMapToJoinConfiguration(gvkmap, true, allowExperimental, true)
+		o, err := documentMapToJoinConfiguration(gvkmap, true, allowExperimental, true, false)
 		if err != nil {
 			return []byte{}, err
 		}
+		if err := mutators.mutate([]any{o}); err != nil {
+			return []byte{}, err
+		}
 		b, err := MarshalKubeadmConfigObject(o, gv)
 		if err != nil {
 			return []byte{}, err
@@ -291,10 +310,13 @@ func MigrateOldConfig(oldConfig []byte, allowExperimental bool) ([]byte, error)
 
 	// Migrate ResetConfiguration if there is any
 	if kubeadmutil.GroupVersionKindsHasResetConfiguration(gvks...) {
-		o, err := documentMapToResetConfiguration(gvkmap, true, allowExperimental, true)
+		o, err := documentMapToResetConfiguration(gvkmap, true, allowExperimental, true, false)
 		if err != nil {
 			return []byte{}, err
 		}
+		if err := mutators.mutate([]any{o}); err != nil {
+			return []byte{}, err
+		}
 		b, err := MarshalKubeadmConfigObject(o, gv)
 		if err != nil {
 			return []byte{}, err
@@ -324,21 +346,21 @@ func ValidateConfig(config []byte, allowExperimental bool) error {
 
 	// Validate InitConfiguration and ClusterConfiguration if there are any in the config
 	if kubeadmutil.GroupVersionKindsHasInitConfiguration(gvks...) || kubeadmutil.GroupVersionKindsHasClusterConfiguration(gvks...) {
-		if _, err := documentMapToInitConfiguration(gvkmap, true, allowExperimental, true); err != nil {
+		if _, err := documentMapToInitConfiguration(gvkmap, true, allowExperimental, true, true); err != nil {
 			return err
 		}
 	}
 
 	// Validate JoinConfiguration if there is any
 	if kubeadmutil.GroupVersionKindsHasJoinConfiguration(gvks...) {
-		if _, err := documentMapToJoinConfiguration(gvkmap, true, allowExperimental, true); err != nil {
+		if _, err := documentMapToJoinConfiguration(gvkmap, true, allowExperimental, true, true); err != nil {
 			return err
 		}
 	}
 
 	// Validate ResetConfiguration if there is any
 	if kubeadmutil.GroupVersionKindsHasResetConfiguration(gvks...) {
-		if _, err := documentMapToResetConfiguration(gvkmap, true, allowExperimental, true); err != nil {
+		if _, err := documentMapToResetConfiguration(gvkmap, true, allowExperimental, true, true); err != nil {
 			return err
 		}
 	}
@@ -362,3 +384,122 @@ func isKubeadmPrereleaseVersion(versionInfo *apimachineryversion.Info, k8sVersio
 	}
 	return false
 }
+
+// prepareStaticVariables takes a given config and stores values from it in variables
+// that can be used from multiple packages.
+func prepareStaticVariables(config any) {
+	switch c := config.(type) {
+	case *kubeadmapi.InitConfiguration:
+		kubeadmapi.SetActiveTimeouts(c.Timeouts)
+	case *kubeadmapi.JoinConfiguration:
+		kubeadmapi.SetActiveTimeouts(c.Timeouts)
+	case *kubeadmapi.ResetConfiguration:
+		kubeadmapi.SetActiveTimeouts(c.Timeouts)
+	case *kubeadmapi.UpgradeConfiguration:
+		kubeadmapi.SetActiveTimeouts(c.Timeouts)
+	}
+}
+
+// migrateMutator can be used to mutate a slice of configuration objects.
+// The mutation is applied in-place and no copies are made.
+type migrateMutator struct {
+	in         []any
+	mutateFunc func(in []any) error
+}
+
+// migrateMutators holds a list of registered mutators.
+type migrateMutators []migrateMutator
+
+// mutate can be called on a list of registered mutators to find a suitable one to perform
+// a configuration object mutation.
+func (mutators migrateMutators) mutate(in []any) error {
+	var mutator *migrateMutator
+	for idx, m := range mutators {
+		if len(m.in) != len(in) {
+			continue
+		}
+		inputMatch := true
+		for idx := range m.in {
+			if reflect.TypeOf(m.in[idx]) != reflect.TypeOf(in[idx]) {
+				inputMatch = false
+				break
+			}
+		}
+		if inputMatch {
+			mutator = &mutators[idx]
+			break
+		}
+	}
+	if mutator == nil {
+		return errors.Errorf("could not find a mutator for input: %#v", in)
+	}
+	return mutator.mutateFunc(in)
+}
+
+// addEmpty adds an empty migrate mutator for a given input.
+func (mutators *migrateMutators) addEmpty(in []any) {
+	mutator := migrateMutator{
+		in:         in,
+		mutateFunc: func(in []any) error { return nil },
+	}
+	*mutators = append(*mutators, mutator)
+}
+
+// defaultMutators returns the default list of mutators for known configuration objects.
+// TODO: make this function return defaultEmptyMutators() when v1beta3 is removed.
+func defaultMigrateMutators() migrateMutators {
+	var (
+		mutators migrateMutators
+		mutator  migrateMutator
+	)
+
+	// mutator for InitConfiguration, ClusterConfiguration.
+	mutator = migrateMutator{
+		in: []any{(*kubeadmapi.InitConfiguration)(nil)},
+		mutateFunc: func(in []any) error {
+			a := in[0].(*kubeadmapi.InitConfiguration)
+			a.Timeouts.ControlPlaneComponentHealthCheck.Duration = a.APIServer.TimeoutForControlPlane.Duration
+			a.APIServer.TimeoutForControlPlane = nil
+			return nil
+		},
+	}
+	mutators = append(mutators, mutator)
+
+	// mutator for JoinConfiguration.
+	mutator = migrateMutator{
+		in: []any{(*kubeadmapi.JoinConfiguration)(nil)},
+		mutateFunc: func(in []any) error {
+			a := in[0].(*kubeadmapi.JoinConfiguration)
+			a.Timeouts.Discovery.Duration = a.Discovery.Timeout.Duration
+			a.Discovery.Timeout = nil
+			return nil
+		},
+	}
+	mutators = append(mutators, mutator)
+
+	// empty mutator for ResetConfiguration.
+	mutators.addEmpty([]any{(*kubeadmapi.ResetConfiguration)(nil)})
+
+	return mutators
+}
+
+// defaultEmptyMigrateMutators returns a list of empty mutators for known types.
+func defaultEmptyMigrateMutators() migrateMutators {
+	mutators := &migrateMutators{}
+
+	mutators.addEmpty([]any{(*kubeadmapi.InitConfiguration)(nil)})
+	mutators.addEmpty([]any{(*kubeadmapi.JoinConfiguration)(nil)})
+	mutators.addEmpty([]any{(*kubeadmapi.ResetConfiguration)(nil)})
+
+	return *mutators
+}
+
+// isKubeadmConfigPresent checks if a kubeadm config type is found in the provided document map
+func isKubeadmConfigPresent(docmap kubeadmapi.DocumentMap) bool {
+	for gvk := range docmap {
+		if gvk.Group == kubeadmapi.GroupName {
+			return true
+		}
+	}
+	return false
+}
