@@ -23,7 +23,7 @@ import (
 	_ "github.com/aojea/sloppy-netparser"
 	_ "github.com/client9/misspell/cmd/misspell"
 	_ "github.com/golangci/golangci-lint/cmd/golangci-lint"
-	_ "github.com/google/go-flow-levee/cmd/levee"
+	_ "github.com/jcchavezs/porto/cmd/porto"
 	_ "honnef.co/go/tools/cmd/staticcheck"
 	_ "sigs.k8s.io/logtools/logcheck"
 
@@ -31,9 +31,9 @@ import (
 	_ "github.com/cespare/prettybench"
 	_ "gotest.tools/gotestsum"
 
-	// dependencies
-	_ "sigs.k8s.io/zeitgeist"
-
 	// mockgen
 	_ "github.com/golang/mock/mockgen"
+
+	// tools like cpu
+	_ "go.uber.org/automaxprocs"
 )
