@@ -17,6 +17,7 @@ limitations under the License.
 package kuberuntime
 
 import (
+	"context"
 	"os"
 	"path/filepath"
 	"testing"
@@ -160,6 +161,7 @@ func TestSandboxGC(t *testing.T) {
 		},
 	} {
 		t.Run(test.description, func(t *testing.T) {
+			ctx := context.Background()
 			podStateProvider.removed = make(map[types.UID]struct{})
 			podStateProvider.terminated = make(map[types.UID]struct{})
 			fakeSandboxes := makeFakePodSandboxes(t, m, test.sandboxes)
@@ -175,13 +177,13 @@ func TestSandboxGC(t *testing.T) {
 			fakeRuntime.SetFakeSandboxes(fakeSandboxes)
 			fakeRuntime.SetFakeContainers(fakeContainers)
 
-			err := m.containerGC.evictSandboxes(test.evictTerminatingPods)
+			err := m.containerGC.evictSandboxes(ctx, test.evictTerminatingPods)
 			assert.NoError(t, err)
-			realRemain, err := fakeRuntime.ListPodSandbox(nil)
+			realRemain, err := fakeRuntime.ListPodSandbox(ctx, nil)
 			assert.NoError(t, err)
 			assert.Len(t, realRemain, len(test.remain))
 			for _, remain := range test.remain {
-				resp, err := fakeRuntime.PodSandboxStatus(fakeSandboxes[remain].Id, false)
+				resp, err := fakeRuntime.PodSandboxStatus(ctx, fakeSandboxes[remain].Id, false)
 				assert.NoError(t, err)
 				assert.Equal(t, &fakeSandboxes[remain].PodSandboxStatus, resp.Status)
 			}
@@ -387,6 +389,7 @@ func TestContainerGC(t *testing.T) {
 		},
 	} {
 		t.Run(test.description, func(t *testing.T) {
+			ctx := context.Background()
 			podStateProvider.removed = make(map[types.UID]struct{})
 			podStateProvider.terminated = make(map[types.UID]struct{})
 			fakeContainers := makeFakeContainers(t, m, test.containers)
@@ -403,13 +406,13 @@ func TestContainerGC(t *testing.T) {
 			if test.policy == nil {
 				test.policy = &defaultGCPolicy
 			}
-			err := m.containerGC.evictContainers(*test.policy, test.allSourcesReady, test.evictTerminatingPods)
+			err := m.containerGC.evictContainers(ctx, *test.policy, test.allSourcesReady, test.evictTerminatingPods)
 			assert.NoError(t, err)
-			realRemain, err := fakeRuntime.ListContainers(nil)
+			realRemain, err := fakeRuntime.ListContainers(ctx, nil)
 			assert.NoError(t, err)
 			assert.Len(t, realRemain, len(test.remain))
 			for _, remain := range test.remain {
-				resp, err := fakeRuntime.ContainerStatus(fakeContainers[remain].Id, false)
+				resp, err := fakeRuntime.ContainerStatus(ctx, fakeContainers[remain].Id, false)
 				assert.NoError(t, err)
 				assert.Equal(t, &fakeContainers[remain].ContainerStatus, resp.Status)
 			}
@@ -419,6 +422,7 @@ func TestContainerGC(t *testing.T) {
 
 // Notice that legacy container symlink is not tested since it may be deprecated soon.
 func TestPodLogDirectoryGC(t *testing.T) {
+	ctx := context.Background()
 	_, _, m, err := createTestRuntimeManager()
 	assert.NoError(t, err)
 	fakeOS := m.osInterface.(*containertest.FakeOS)
@@ -426,10 +430,11 @@ func TestPodLogDirectoryGC(t *testing.T) {
 
 	// pod log directories without corresponding pods should be removed.
 	files := []string{"123", "456", "789", "012", "name_namespace_321", "name_namespace_654"}
+	podLogsDirectory := "/var/log/pods"
 	removed := []string{
-		filepath.Join(podLogsRootDirectory, "789"),
-		filepath.Join(podLogsRootDirectory, "012"),
-		filepath.Join(podLogsRootDirectory, "name_namespace_654"),
+		filepath.Join(podLogsDirectory, "789"),
+		filepath.Join(podLogsDirectory, "012"),
+		filepath.Join(podLogsDirectory, "name_namespace_654"),
 	}
 	podStateProvider.removed["012"] = struct{}{}
 	podStateProvider.removed["789"] = struct{}{}
@@ -438,29 +443,30 @@ func TestPodLogDirectoryGC(t *testing.T) {
 	ctrl := gomock.NewController(t)
 	defer ctrl.Finish()
 
-	fakeOS.ReadDirFn = func(string) ([]os.FileInfo, error) {
-		var fileInfos []os.FileInfo
+	fakeOS.ReadDirFn = func(string) ([]os.DirEntry, error) {
+		var dirEntries []os.DirEntry
 		for _, file := range files {
-			mockFI := containertest.NewMockFileInfo(ctrl)
-			mockFI.EXPECT().Name().Return(file)
-			fileInfos = append(fileInfos, mockFI)
+			mockDE := containertest.NewMockDirEntry(ctrl)
+			mockDE.EXPECT().Name().Return(file)
+			dirEntries = append(dirEntries, mockDE)
 		}
-		return fileInfos, nil
+		return dirEntries, nil
 	}
 
 	// allSourcesReady == true, pod log directories without corresponding pod should be removed.
-	err = m.containerGC.evictPodLogsDirectories(true)
+	err = m.containerGC.evictPodLogsDirectories(ctx, true)
 	assert.NoError(t, err)
 	assert.Equal(t, removed, fakeOS.Removes)
 
 	// allSourcesReady == false, pod log directories should not be removed.
 	fakeOS.Removes = []string{}
-	err = m.containerGC.evictPodLogsDirectories(false)
+	err = m.containerGC.evictPodLogsDirectories(ctx, false)
 	assert.NoError(t, err)
 	assert.Empty(t, fakeOS.Removes)
 }
 
 func TestUnknownStateContainerGC(t *testing.T) {
+	ctx := context.Background()
 	fakeRuntime, _, m, err := createTestRuntimeManager()
 	assert.NoError(t, err)
 
@@ -472,13 +478,13 @@ func TestUnknownStateContainerGC(t *testing.T) {
 	})
 	fakeRuntime.SetFakeContainers(fakeContainers)
 
-	err = m.containerGC.evictContainers(defaultGCPolicy, true, false)
+	err = m.containerGC.evictContainers(ctx, defaultGCPolicy, true, false)
 	assert.NoError(t, err)
 
 	assert.Contains(t, fakeRuntime.GetCalls(), "StopContainer", "RemoveContainer",
 		"container in unknown state should be stopped before being removed")
 
-	remain, err := fakeRuntime.ListContainers(nil)
+	remain, err := fakeRuntime.ListContainers(ctx, nil)
 	assert.NoError(t, err)
 	assert.Empty(t, remain)
 }
