@@ -22,7 +22,7 @@ import (
 	"strconv"
 	"testing"
 
-	"k8s.io/gengo/types"
+	"k8s.io/gengo/v2/types"
 	"k8s.io/klog/v2"
 )
 
