@@ -159,7 +159,7 @@ __EOF__
   local tries=5
   for i in $(seq 1 $tries); do
       local output
-      output=$(kubectl "${kube_flags[@]:?}" api-resources --api-group mygroup.example.com -oname)
+      output=$(kubectl "${kube_flags[@]:?}" api-resources --api-group mygroup.example.com -oname || true)
       if kube::test::if_has_string "$output" resources.mygroup.example.com; then
           break
       fi
@@ -231,18 +231,18 @@ __EOF__
   kube::test::get_object_assert pods "{{range.items}}{{${id_field:?}}}:{{end}}" ''
   kubectl delete pvc b-pvc 2>&1 "${kube_flags[@]:?}"
 
-  ## kubectl apply --prune --prune-whitelist
+  ## kubectl apply --prune --prune-allowlist
   # Pre-Condition: no POD exists
   kube::test::get_object_assert pods "{{range.items}}{{${id_field:?}}}:{{end}}" ''
   # apply pod a
   kubectl apply --prune -l prune-group=true -f hack/testdata/prune/a.yaml "${kube_flags[@]:?}"
   # check right pod exists
   kube::test::get_object_assert 'pods a' "{{${id_field:?}}}" 'a'
-  # apply svc and don't prune pod a by overwriting whitelist
-  kubectl apply --prune -l prune-group=true -f hack/testdata/prune/svc.yaml --prune-whitelist core/v1/Service 2>&1 "${kube_flags[@]:?}"
+  # apply svc and don't prune pod a by overwriting allowlist
+  kubectl apply --prune -l prune-group=true -f hack/testdata/prune/svc.yaml --prune-allowlist core/v1/Service 2>&1 "${kube_flags[@]:?}"
   kube::test::get_object_assert 'service prune-svc' "{{${id_field:?}}}" 'prune-svc'
   kube::test::get_object_assert 'pods a' "{{${id_field:?}}}" 'a'
-  # apply svc and prune pod a with default whitelist
+  # apply svc and prune pod a with default allowlist
   kubectl apply --prune -l prune-group=true -f hack/testdata/prune/svc.yaml 2>&1 "${kube_flags[@]:?}"
   kube::test::get_object_assert 'service prune-svc' "{{${id_field:?}}}" 'prune-svc'
   kube::test::get_object_assert pods "{{range.items}}{{${id_field:?}}}:{{end}}" ''
