@@ -44,7 +44,7 @@ import (
 * runtime, but it will not be present in the config, thus making the pod a
 * "unknown pod". Kubelet should then proceed to terminate these unknown pods.
  */
-var _ = SIGDescribe("Unknown Pods [Serial] [Disruptive]", func() {
+var _ = SIGDescribe("Unknown Pods", framework.WithSerial(), framework.WithDisruptive(), func() {
 	f := framework.NewDefaultFramework("unknown-pods")
 	f.NamespacePodSecurityLevel = admissionapi.LevelBaseline
 
