@@ -31,7 +31,6 @@ import (
 	restclient "k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/cache"
 	fakecloud "k8s.io/cloud-provider/fake"
-	"k8s.io/klog/v2/ktesting"
 	kubeapiservertesting "k8s.io/kubernetes/cmd/kube-apiserver/app/testing"
 	"k8s.io/kubernetes/pkg/controller/volume/attachdetach"
 	volumecache "k8s.io/kubernetes/pkg/controller/volume/attachdetach/cache"
@@ -41,6 +40,7 @@ import (
 	volumetest "k8s.io/kubernetes/pkg/volume/testing"
 	"k8s.io/kubernetes/pkg/volume/util"
 	"k8s.io/kubernetes/test/integration/framework"
+	"k8s.io/kubernetes/test/utils/ktesting"
 )
 
 func fakePodWithVol(namespace string) *v1.Pod {
@@ -120,7 +120,7 @@ func fakePodWithPVC(name, pvcName, namespace string) (*v1.Pod, *v1.PersistentVol
 			AccessModes: []v1.PersistentVolumeAccessMode{
 				v1.ReadWriteOnce,
 			},
-			Resources: v1.ResourceRequirements{
+			Resources: v1.VolumeResourceRequirements{
 				Requests: v1.ResourceList{
 					v1.ResourceName(v1.ResourceStorage): resource.MustParse("5Gi"),
 				},
@@ -156,36 +156,35 @@ func TestPodDeletionWithDswp(t *testing.T) {
 		},
 	}
 
-	testClient, ctrl, pvCtrl, informers := createAdClients(t, server, defaultSyncPeriod, defaultTimerConfig)
+	tCtx := ktesting.Init(t)
+	defer tCtx.Cancel("test has completed")
+	testClient, ctrl, pvCtrl, informers := createAdClients(tCtx, t, server, defaultSyncPeriod, defaultTimerConfig)
 
 	ns := framework.CreateNamespaceOrDie(testClient, namespaceName, t)
 	defer framework.DeleteNamespaceOrDie(testClient, ns, t)
 
 	pod := fakePodWithVol(namespaceName)
 
-	if _, err := testClient.CoreV1().Nodes().Create(context.TODO(), node, metav1.CreateOptions{}); err != nil {
+	if _, err := testClient.CoreV1().Nodes().Create(tCtx, node, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to created node : %v", err)
 	}
 
 	// start controller loop
-	ctx, cancel := context.WithCancel(context.Background())
-	defer cancel()
-
-	go informers.Core().V1().Nodes().Informer().Run(ctx.Done())
-	if _, err := testClient.CoreV1().Pods(ns.Name).Create(context.TODO(), pod, metav1.CreateOptions{}); err != nil {
+	go informers.Core().V1().Nodes().Informer().Run(tCtx.Done())
+	if _, err := testClient.CoreV1().Pods(ns.Name).Create(tCtx, pod, metav1.CreateOptions{}); err != nil {
 		t.Errorf("Failed to create pod : %v", err)
 	}
 
 	podInformer := informers.Core().V1().Pods().Informer()
-	go podInformer.Run(ctx.Done())
+	go podInformer.Run(tCtx.Done())
 
-	go informers.Core().V1().PersistentVolumeClaims().Informer().Run(ctx.Done())
-	go informers.Core().V1().PersistentVolumes().Informer().Run(ctx.Done())
-	go informers.Storage().V1().VolumeAttachments().Informer().Run(ctx.Done())
-	initCSIObjects(ctx.Done(), informers)
-	go ctrl.Run(ctx)
+	go informers.Core().V1().PersistentVolumeClaims().Informer().Run(tCtx.Done())
+	go informers.Core().V1().PersistentVolumes().Informer().Run(tCtx.Done())
+	go informers.Storage().V1().VolumeAttachments().Informer().Run(tCtx.Done())
+	initCSIObjects(tCtx.Done(), informers)
+	go ctrl.Run(tCtx)
 	// Run pvCtrl to avoid leaking goroutines started during its creation.
-	go pvCtrl.Run(ctx)
+	go pvCtrl.Run(tCtx)
 
 	waitToObservePods(t, podInformer, 1)
 	podKey, err := cache.MetaNamespaceKeyFunc(pod)
@@ -231,7 +230,9 @@ func TestPodUpdateWithWithADC(t *testing.T) {
 		},
 	}
 
-	testClient, ctrl, pvCtrl, informers := createAdClients(t, server, defaultSyncPeriod, defaultTimerConfig)
+	tCtx := ktesting.Init(t)
+	defer tCtx.Cancel("test has completed")
+	testClient, ctrl, pvCtrl, informers := createAdClients(tCtx, t, server, defaultSyncPeriod, defaultTimerConfig)
 
 	ns := framework.CreateNamespaceOrDie(testClient, namespaceName, t)
 	defer framework.DeleteNamespaceOrDie(testClient, ns, t)
@@ -254,16 +255,13 @@ func TestPodUpdateWithWithADC(t *testing.T) {
 	go podInformer.Run(podStopCh)
 
 	// start controller loop
-	ctx, cancel := context.WithCancel(context.Background())
-	defer cancel()
-
-	go informers.Core().V1().PersistentVolumeClaims().Informer().Run(ctx.Done())
-	go informers.Core().V1().PersistentVolumes().Informer().Run(ctx.Done())
-	go informers.Storage().V1().VolumeAttachments().Informer().Run(ctx.Done())
-	initCSIObjects(ctx.Done(), informers)
-	go ctrl.Run(ctx)
+	go informers.Core().V1().PersistentVolumeClaims().Informer().Run(tCtx.Done())
+	go informers.Core().V1().PersistentVolumes().Informer().Run(tCtx.Done())
+	go informers.Storage().V1().VolumeAttachments().Informer().Run(tCtx.Done())
+	initCSIObjects(tCtx.Done(), informers)
+	go ctrl.Run(tCtx)
 	// Run pvCtrl to avoid leaking goroutines started during its creation.
-	go pvCtrl.Run(ctx)
+	go pvCtrl.Run(tCtx)
 
 	waitToObservePods(t, podInformer, 1)
 	podKey, err := cache.MetaNamespaceKeyFunc(pod)
@@ -304,7 +302,9 @@ func TestPodUpdateWithKeepTerminatedPodVolumes(t *testing.T) {
 		},
 	}
 
-	testClient, ctrl, pvCtrl, informers := createAdClients(t, server, defaultSyncPeriod, defaultTimerConfig)
+	tCtx := ktesting.Init(t)
+	defer tCtx.Cancel("test has completed")
+	testClient, ctrl, pvCtrl, informers := createAdClients(tCtx, t, server, defaultSyncPeriod, defaultTimerConfig)
 
 	ns := framework.CreateNamespaceOrDie(testClient, namespaceName, t)
 	defer framework.DeleteNamespaceOrDie(testClient, ns, t)
@@ -327,16 +327,13 @@ func TestPodUpdateWithKeepTerminatedPodVolumes(t *testing.T) {
 	go podInformer.Run(podStopCh)
 
 	// start controller loop
-	ctx, cancel := context.WithCancel(context.Background())
-	defer cancel()
-
-	go informers.Core().V1().PersistentVolumeClaims().Informer().Run(ctx.Done())
-	go informers.Core().V1().PersistentVolumes().Informer().Run(ctx.Done())
-	go informers.Storage().V1().VolumeAttachments().Informer().Run(ctx.Done())
-	initCSIObjects(ctx.Done(), informers)
-	go ctrl.Run(ctx)
+	go informers.Core().V1().PersistentVolumeClaims().Informer().Run(tCtx.Done())
+	go informers.Core().V1().PersistentVolumes().Informer().Run(tCtx.Done())
+	go informers.Storage().V1().VolumeAttachments().Informer().Run(tCtx.Done())
+	initCSIObjects(tCtx.Done(), informers)
+	go ctrl.Run(tCtx)
 	// Run pvCtrl to avoid leaking goroutines started during its creation.
-	go pvCtrl.Run(ctx)
+	go pvCtrl.Run(tCtx)
 
 	waitToObservePods(t, podInformer, 1)
 	podKey, err := cache.MetaNamespaceKeyFunc(pod)
@@ -402,7 +399,7 @@ func waitForPodFuncInDSWP(t *testing.T, dswp volumecache.DesiredStateOfWorld, ch
 	}
 }
 
-func createAdClients(t *testing.T, server *kubeapiservertesting.TestServer, syncPeriod time.Duration, timers attachdetach.TimerConfig) (*clientset.Clientset, attachdetach.AttachDetachController, *persistentvolume.PersistentVolumeController, clientgoinformers.SharedInformerFactory) {
+func createAdClients(ctx context.Context, t *testing.T, server *kubeapiservertesting.TestServer, syncPeriod time.Duration, timers attachdetach.TimerConfig) (*clientset.Clientset, attachdetach.AttachDetachController, *persistentvolume.PersistentVolumeController, clientgoinformers.SharedInformerFactory) {
 	config := restclient.CopyConfig(server.ClientConfig)
 	config.QPS = 1000000
 	config.Burst = 1000000
@@ -425,9 +422,8 @@ func createAdClients(t *testing.T, server *kubeapiservertesting.TestServer, sync
 	plugins := []volume.VolumePlugin{plugin}
 	cloud := &fakecloud.Cloud{}
 	informers := clientgoinformers.NewSharedInformerFactory(testClient, resyncPeriod)
-	logger, ctx := ktesting.NewTestContext(t)
 	ctrl, err := attachdetach.NewAttachDetachController(
-		logger,
+		ctx,
 		testClient,
 		informers.Core().V1().Pods(),
 		informers.Core().V1().Nodes(),
@@ -441,6 +437,7 @@ func createAdClients(t *testing.T, server *kubeapiservertesting.TestServer, sync
 		nil, /* prober */
 		false,
 		5*time.Second,
+		false,
 		timers,
 	)
 
@@ -487,7 +484,10 @@ func TestPodAddedByDswp(t *testing.T) {
 			},
 		},
 	}
-	testClient, ctrl, pvCtrl, informers := createAdClients(t, server, defaultSyncPeriod, defaultTimerConfig)
+
+	tCtx := ktesting.Init(t)
+	defer tCtx.Cancel("test has completed")
+	testClient, ctrl, pvCtrl, informers := createAdClients(tCtx, t, server, defaultSyncPeriod, defaultTimerConfig)
 
 	ns := framework.CreateNamespaceOrDie(testClient, namespaceName, t)
 	defer framework.DeleteNamespaceOrDie(testClient, ns, t)
@@ -495,13 +495,13 @@ func TestPodAddedByDswp(t *testing.T) {
 	pod := fakePodWithVol(namespaceName)
 	podStopCh := make(chan struct{})
 
-	if _, err := testClient.CoreV1().Nodes().Create(context.TODO(), node, metav1.CreateOptions{}); err != nil {
+	if _, err := testClient.CoreV1().Nodes().Create(tCtx, node, metav1.CreateOptions{}); err != nil {
 		t.Fatalf("Failed to created node : %v", err)
 	}
 
 	go informers.Core().V1().Nodes().Informer().Run(podStopCh)
 
-	if _, err := testClient.CoreV1().Pods(ns.Name).Create(context.TODO(), pod, metav1.CreateOptions{}); err != nil {
+	if _, err := testClient.CoreV1().Pods(ns.Name).Create(tCtx, pod, metav1.CreateOptions{}); err != nil {
 		t.Errorf("Failed to create pod : %v", err)
 	}
 
@@ -509,17 +509,13 @@ func TestPodAddedByDswp(t *testing.T) {
 	go podInformer.Run(podStopCh)
 
 	// start controller loop
-	_, ctx := ktesting.NewTestContext(t)
-	ctx, cancel := context.WithCancel(ctx)
-	defer cancel()
-
-	go informers.Core().V1().PersistentVolumeClaims().Informer().Run(ctx.Done())
-	go informers.Core().V1().PersistentVolumes().Informer().Run(ctx.Done())
-	go informers.Storage().V1().VolumeAttachments().Informer().Run(ctx.Done())
-	initCSIObjects(ctx.Done(), informers)
-	go ctrl.Run(ctx)
+	go informers.Core().V1().PersistentVolumeClaims().Informer().Run(tCtx.Done())
+	go informers.Core().V1().PersistentVolumes().Informer().Run(tCtx.Done())
+	go informers.Storage().V1().VolumeAttachments().Informer().Run(tCtx.Done())
+	initCSIObjects(tCtx.Done(), informers)
+	go ctrl.Run(tCtx)
 	// Run pvCtrl to avoid leaking goroutines started during its creation.
-	go pvCtrl.Run(ctx)
+	go pvCtrl.Run(tCtx)
 
 	waitToObservePods(t, podInformer, 1)
 	podKey, err := cache.MetaNamespaceKeyFunc(pod)
@@ -555,9 +551,13 @@ func TestPVCBoundWithADC(t *testing.T) {
 	// Disable ServiceAccount admission plugin as we don't have serviceaccount controller running.
 	server := kubeapiservertesting.StartTestServerOrDie(t, nil, []string{"--disable-admission-plugins=ServiceAccount"}, framework.SharedEtcd())
 	defer server.TearDownFn()
+
+	tCtx := ktesting.Init(t)
+	defer tCtx.Cancel("test has completed")
+
 	namespaceName := "test-pod-deletion"
 
-	testClient, ctrl, pvCtrl, informers := createAdClients(t, server, defaultSyncPeriod, attachdetach.TimerConfig{
+	testClient, ctrl, pvCtrl, informers := createAdClients(tCtx, t, server, defaultSyncPeriod, attachdetach.TimerConfig{
 		ReconcilerLoopPeriod:                        100 * time.Millisecond,
 		ReconcilerMaxWaitForUnmountDuration:         6 * time.Second,
 		DesiredStateOfWorldPopulatorLoopSleepPeriod: 24 * time.Hour,
@@ -600,14 +600,11 @@ func TestPVCBoundWithADC(t *testing.T) {
 	}
 
 	// start controller loop
-	ctx, cancel := context.WithCancel(context.Background())
-	defer cancel()
-
-	informers.Start(ctx.Done())
-	informers.WaitForCacheSync(ctx.Done())
-	initCSIObjects(ctx.Done(), informers)
-	go ctrl.Run(ctx)
-	go pvCtrl.Run(ctx)
+	informers.Start(tCtx.Done())
+	informers.WaitForCacheSync(tCtx.Done())
+	initCSIObjects(tCtx.Done(), informers)
+	go ctrl.Run(tCtx)
+	go pvCtrl.Run(tCtx)
 
 	waitToObservePods(t, informers.Core().V1().Pods().Informer(), 4)
 	// Give attachdetach controller enough time to populate pods into DSWP.
